@@ -26,9 +26,10 @@
 #endif
 
 #include <stdlib.h>
+#include <string.h>
 
-#include <gnome.h>
-#include <bonobo.h>
+#include <glib.h>
+#include <gdk/gdk.h>
 
 #include "preferences.h"
 
@@ -45,19 +46,18 @@ static orientation_t read_orientation_from_string  (gchar *string);
 static wallpaper_type_t read_wptype_from_string (gchar *string);
 
 static GEnumValue _bg_wptype_values[] = {
-	{ WPTYPE_TILED, "wallpaper", N_("Tiled") },
-	{ WPTYPE_CENTERED, "centered", N_("Centered") },
-	{ WPTYPE_SCALED, "scaled", N_("Scaled") },
-	{ WPTYPE_STRETCHED, "stretched", N_("Stretched") },
-	{ WPTYPE_EMBOSSED, "embossed", N_("Embossed") },
-	{ WPTYPE_NONE, "none", N_("None") },
+	{ WPTYPE_TILED, "WPTYPE_TILED", "wallpaper"},
+	{ WPTYPE_CENTERED, "WPTYPE_CENTERED", "centered"},
+	{ WPTYPE_SCALED, "WPTYPE_SCALED", "scaled"},
+	{ WPTYPE_STRETCHED, "WPTYPE_STRETCHED", "stretched"},
+	{ WPTYPE_NONE, "WPTYPE_NONE", "none"},
 	{ 0, NULL, NULL }
 };
 
 static GEnumValue _bg_orientation_values[] = {
-	{ ORIENTATION_SOLID, "solid", N_("Solid") },
-	{ ORIENTATION_HORIZ, "horizontal-gradient", N_("Horizontal Gradient") },
-	{ ORIENTATION_VERT, "vertical-gradient", N_("Vertical Gradient") },
+	{ ORIENTATION_SOLID, "ORIENTATION_SOLID", "solid"},
+	{ ORIENTATION_HORIZ, "ORIENTATION_HORIZ", "horizontal-gradient"},
+	{ ORIENTATION_VERT, "ORIENTATION_VERT", "vertical-gradient"},
 	{ 0, NULL, NULL }
 };
 
@@ -178,12 +178,22 @@ bg_preferences_clone (const BGPreferences *prefs)
 	new_prefs->orientation        = prefs->orientation;
 	new_prefs->wallpaper_type     = prefs->wallpaper_type;
 
-	if (prefs->color1)
-		new_prefs->color1 = gdk_color_copy (prefs->color1);
-	if (prefs->color2)
-		new_prefs->color2 = gdk_color_copy (prefs->color2);
+	if (new_prefs->color1 != NULL)
+		gdk_color_free (new_prefs->color1);
+	new_prefs->color1 = (prefs->color1 != NULL)
+		? gdk_color_copy (prefs->color1) : NULL;
 
+	if (new_prefs->color2 != NULL)
+		gdk_color_free (new_prefs->color2);
+	new_prefs->color2 = (prefs->color2 != NULL)
+		? gdk_color_copy (prefs->color2) : NULL;
+
+	if (new_prefs->wallpaper_filename != NULL)
+		g_free (new_prefs->wallpaper_filename);
 	new_prefs->wallpaper_filename = g_strdup (prefs->wallpaper_filename);
+
+	if (new_prefs->wallpaper_sel_path != NULL)
+		g_free (new_prefs->wallpaper_sel_path);
 	new_prefs->wallpaper_sel_path = g_strdup (prefs->wallpaper_sel_path);;
 
 	new_prefs->auto_apply         = prefs->auto_apply;
@@ -204,8 +214,20 @@ bg_preferences_finalize (GObject *object)
 	prefs = BG_PREFERENCES (object);
 
 	g_free (prefs->wallpaper_filename);
-	g_free (prefs->wallpaper_sel_path);
+	prefs->wallpaper_filename = NULL;
 
+	g_free (prefs->wallpaper_sel_path);
+	prefs->wallpaper_sel_path = NULL;
+
+	if (prefs->color1 != NULL) {
+		gdk_color_free (prefs->color1);
+		prefs->color1 = NULL;
+	}
+	if (prefs->color2 != NULL) {
+		gdk_color_free (prefs->color2);
+		prefs->color2 = NULL;
+	}
+	
 	parent_class->finalize (object);
 }
 
@@ -214,7 +236,7 @@ bg_preferences_load (BGPreferences *prefs)
 {
 	GConfClient *client;
 	GError      *error = NULL;
-
+	char *tmp;
 	g_return_if_fail (prefs != NULL);
 	g_return_if_fail (IS_BG_PREFERENCES (prefs));
 
@@ -223,9 +245,18 @@ bg_preferences_load (BGPreferences *prefs)
 	prefs->enabled = gconf_client_get_bool (client, BG_PREFERENCES_DRAW_BACKGROUND, &error);
 	prefs->wallpaper_filename = gconf_client_get_string (client, BG_PREFERENCES_PICTURE_FILENAME, &error);
 
-	prefs->color1 = read_color_from_string (gconf_client_get_string (client, BG_PREFERENCES_PRIMARY_COLOR, &error));
-	prefs->color2 = read_color_from_string (gconf_client_get_string (client, BG_PREFERENCES_SECONDARY_COLOR, &error));
+	if (prefs->color1 != NULL)
+		gdk_color_free (prefs->color1);
+	tmp = gconf_client_get_string (client, BG_PREFERENCES_PRIMARY_COLOR, &error);
+	prefs->color1 = read_color_from_string (tmp);
+	g_free (tmp);
 
+	if (prefs->color2 != NULL)
+		gdk_color_free (prefs->color2);
+	tmp = gconf_client_get_string (client, BG_PREFERENCES_SECONDARY_COLOR, &error);
+	prefs->color2 = read_color_from_string (tmp);
+	g_free (tmp);
+	
 	prefs->opacity = gconf_client_get_int (client, BG_PREFERENCES_PICTURE_OPACITY, &error);
 	if (prefs->opacity >= 100 || prefs->opacity < 0)
 		prefs->adjust_opacity = FALSE;
@@ -244,6 +275,8 @@ bg_preferences_load (BGPreferences *prefs)
 	} else {
 	  prefs->wallpaper_enabled = TRUE;
 	}
+
+	g_object_unref (client);
 }
 
 /* Parse the event name given (the event being notification of a property having
@@ -280,9 +313,13 @@ bg_preferences_merge_entry (BGPreferences    *prefs,
 			prefs->wallpaper_enabled = FALSE;
 	}
 	else if (!strcmp (entry->key, BG_PREFERENCES_PRIMARY_COLOR)) {
+		if (prefs->color1 != NULL)
+			gdk_color_free (prefs->color1);
 		prefs->color1 = read_color_from_string (gconf_value_get_string (value));
 	}
 	else if (!strcmp (entry->key, BG_PREFERENCES_SECONDARY_COLOR)) {
+		if (prefs->color2 != NULL)
+			gdk_color_free (prefs->color2);
 		prefs->color2 = read_color_from_string (gconf_value_get_string (value));
 	}
 	else if (!strcmp (entry->key, BG_PREFERENCES_PICTURE_OPACITY)) {
@@ -326,8 +363,6 @@ read_wptype_from_string (gchar *string)
 			type =  WPTYPE_SCALED;
 		} else if (!strncmp (string, "stretched", sizeof ("stretched"))) {
 			type =  WPTYPE_STRETCHED;
-		} else if (!strncmp (string, "embossed", sizeof ("embossed"))) {
-			type =  WPTYPE_EMBOSSED;
 		}
 		g_free (string);
 	}
@@ -355,20 +390,13 @@ read_orientation_from_string (gchar *string)
 static GdkColor *
 read_color_from_string (const gchar *string) 
 {
-	GdkColor *color;
-	gint32 rgb;
+	GdkColor color;
 
-	color = g_new0 (GdkColor, 1);
-
-	if (string != NULL) {
-		gdk_color_parse (string, color);
-		rgb = ((color->red >> 8) << 16) ||
-			((color->green >> 8) << 8) ||
-			(color->blue >> 8);
-		gdk_rgb_find_color (gdk_rgb_get_colormap (), color);
-	}
-
-	return color;
+	/* If all else fails use black */
+	if (string == NULL || !gdk_color_parse (string, &color))
+		gdk_color_parse ("black", &color);
+	gdk_rgb_find_color (gdk_rgb_get_colormap (), &color);
+	return gdk_color_copy (&color);
 }
 
 const gchar*
@@ -384,8 +412,6 @@ bg_preferences_get_wptype_as_string (wallpaper_type_t wp)
 			return "scaled";
 		case WPTYPE_STRETCHED:
 			return "stretched";
-		case WPTYPE_EMBOSSED:
-			return "embossed";
 		case WPTYPE_NONE:
 			return "none";
 	        case WPTYPE_UNSET:
@@ -416,10 +442,13 @@ bg_preferences_save (BGPreferences *prefs)
 {
 	GConfChangeSet *cs;
 	gchar *tmp;
+	GConfClient *client;
 	
 	g_return_if_fail (prefs != NULL);
 	g_return_if_fail (IS_BG_PREFERENCES (prefs));
 
+	client = gconf_client_get_default();
+	
 	cs = gconf_change_set_new ();
 	gconf_change_set_set_bool (cs, BG_PREFERENCES_DRAW_BACKGROUND, prefs->enabled);
 	if (prefs->wallpaper_enabled)
@@ -445,8 +474,9 @@ bg_preferences_save (BGPreferences *prefs)
 
 	gconf_change_set_set_string (cs, BG_PREFERENCES_COLOR_SHADING_TYPE, bg_preferences_get_orientation_as_string (prefs->orientation));
 
-	gconf_client_commit_change_set (gconf_client_get_default (), cs, TRUE, NULL);
+	gconf_client_commit_change_set (client, cs, TRUE, NULL);
 	gconf_change_set_unref (cs);
+	g_object_unref (client);
 }
 
 
