@@ -42,7 +42,6 @@
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-url.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <stdlib.h>
@@ -367,14 +366,11 @@ add_startup_timeout (GdkScreen         *screen,
  * this should not be needed.
  */
 static Time
-slowly_and_stupidly_obtain_timestamp (SnDisplay *display)
+slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 {
 	Window xwindow;
-	Display *xdisplay;
 	XEvent event;
 	
-	xdisplay = sn_display_get_x_display (display);
-	
 	{
 		XSetWindowAttributes attrs;
 		Atom atom_name;
@@ -473,9 +469,7 @@ void nautilus_launch_show_file (NautilusFile *file,
 	
 	/* Only initiate notification if application supports it. */
 	if (application) {
-		startup_notify = gnome_vfs_application_registry_get_bool_value (application->id,
-										GNOME_VFS_APPLICATION_REGISTRY_STARTUP_NOTIFY,
-										NULL);
+		startup_notify = gnome_vfs_mime_application_supports_startup_notification (application);
 	} else {
 		startup_notify = FALSE;
 	}
@@ -514,9 +508,9 @@ void nautilus_launch_show_file (NautilusFile *file,
 			char **old_envp;
 			Time timestamp;
 
-			timestamp = slowly_and_stupidly_obtain_timestamp (sn_display);
+			timestamp = slowly_and_stupidly_obtain_timestamp (GDK_WINDOW_XDISPLAY (GTK_WIDGET (parent_window)->window));
 
-			binary_name = application->command;
+			binary_name = gnome_vfs_mime_application_get_binary_name (application);
 		
 			sn_launcher_context_set_binary_name (sn_context,
 							     binary_name);
@@ -711,9 +705,8 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 
 	
 	/* Only initiate notification if application supports it. */
-	if (gnome_vfs_application_registry_get_bool_value (application->id, 
-							   GNOME_VFS_APPLICATION_REGISTRY_STARTUP_NOTIFY,
-							   NULL)) {
+	if (gnome_vfs_mime_application_supports_startup_notification (application))
+	{ 
 		char *name;
 		char *icon;
 
@@ -746,9 +739,9 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 			char **old_envp;
 			Time timestamp;
 
-			timestamp = slowly_and_stupidly_obtain_timestamp (sn_display);
+			timestamp = slowly_and_stupidly_obtain_timestamp (GDK_WINDOW_XDISPLAY (GTK_WIDGET (parent_window)->window));
 			
-			binary_name = application->command;
+			binary_name = gnome_vfs_mime_application_get_binary_name (application);
 		
 			sn_launcher_context_set_binary_name (sn_context,
 							     binary_name);
@@ -862,6 +855,9 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 	const GList *p;
 	int total, count;
 	char **envp;
+#ifdef HAVE_STARTUP_NOTIFICATION
+	Time timestamp;
+#endif
 
 	/* strip the leading command specifier */
 	if (eel_str_has_prefix (desktop_file_uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
@@ -951,6 +947,11 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 	}
 
 	error = NULL;
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+	timestamp = slowly_and_stupidly_obtain_timestamp (GDK_WINDOW_XDISPLAY (GTK_WIDGET (parent_window)->window));
+	gnome_desktop_item_set_launch_time (ditem, timestamp);
+#endif
 	gnome_desktop_item_launch_with_env (ditem, (GList *) parameter_uris,
 					    flags, envp,
 					    &error);
